{% load floppyforms %}{% block row %}{% for field in fields %}
{% with classes=field.css_classes label=label|default:field.label help_text=help_text|default:field.help_text %}
{% block field %}
{% block errors %}{% include "floppyforms/errors.html" with errors=field.errors %}{% endblock %}
<p{% if classes %} class="{{ classes }}"{% endif %}>
    {% block label %}{% if field|id %}<label for="{{ field|id }}">{% endif %}{{ label }}{% if label|last not in ".:!?" %}:{% endif %}{% if field|id %}</label>{% endif %}{% endblock %}
    {% block widget %}{% formfield field %}{% endblock %}
    {% block help_text %}{% if help_text %}<span class="helptext">{{ help_text }}</span>{% endif %}{% endblock %}
    {% block hidden_fields %}{% for field in hidden_fields %}{{ field.as_hidden }}{% endfor %}{% endblock %}
</p>{% endblock %}
{% endwith %}{% endfor %}{% endblock %}
